@@ -59,6 +59,7 @@ pub enum QueueErrorKind {
 /// Errors coming from the reactor.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum ReactorErrorKind {
+    /// Indicates an incorrect source type.
     IncorrectSourceType,
 }
 
@@ -73,6 +74,7 @@ impl fmt::Display for ReactorErrorKind {
 /// Error types that can be created by the executor.
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum ExecutorErrorKind {
+    /// Error variants for executor queues.
     QueueError {
         /// index of the queue
         index: usize,
@@ -97,6 +99,52 @@ impl fmt::Display for ExecutorErrorKind {
     }
 }
 
+/// Error types that can be created when building executors.
+#[derive(Debug)]
+pub enum BuilderErrorKind {
+    /// Error type for using a [`Placement`](crate::Placement) that requires
+    /// more CPUs than available.
+    InsufficientCpus {
+        /// The number of CPUs required for success.
+        required: usize,
+
+        /// The number of CPUs available.
+        available: usize,
+    },
+    /// Error type for using [`Placement::Custom`](crate::Placement::Custom)
+    /// with a number of [`CpuSet`](crate::CpuSet)s that does not match the
+    /// number of shards requested.
+    NrShards {
+        /// The number of [`CpuSet`](crate::CpuSet)s provided.
+        cpu_sets: usize,
+
+        /// The number of shards requested.
+        shards: usize,
+    },
+    /// Error type returned by
+    /// [`PoolThreadHandles::join_all`](crate::PoolThreadHandles::join_all)
+    /// for threads that panicked.  The contained error is forwarded from
+    /// [`JoinHandle`](std::thread::JoinHandle).
+    ThreadPanic(Box<dyn std::any::Any + Send>),
+}
+
+impl fmt::Display for BuilderErrorKind {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::InsufficientCpus {
+                available,
+                required,
+            } => write!(f, "found {} of {} required CPUs", available, required),
+            Self::NrShards { cpu_sets, shards } => write!(
+                f,
+                "provided {} CpuSet(s), requested {} shards",
+                cpu_sets, shards
+            ),
+            Self::ThreadPanic(_) => write!(f, "thread panicked"),
+        }
+    }
+}
+
 /// Composite error type to encompass all error types glommio produces.
 ///
 /// Single error type that will be produced by any public Glommio API
@@ -136,10 +184,10 @@ pub enum GlommioError<T> {
         /// The operation that was being attempted.
         op: &'static str,
 
-        /// The path of the file, if relavent.
+        /// The path of the file, if relevant.
         path: Option<PathBuf>,
 
-        /// The numeric file descriptor of the relavent resource.
+        /// The numeric file descriptor of the relevant resource.
         fd: Option<RawFd>,
     },
 
@@ -147,6 +195,9 @@ pub enum GlommioError<T> {
     /// inside of the executor.
     ExecutorError(ExecutorErrorKind),
 
+    /// Error variant(s) produced when building executors.
+    BuilderError(BuilderErrorKind),
+
     /// The resource in question is closed. Generic because the channel
     /// variant needs to return the actual item sent into the channel.
     Closed(ResourceType<T>),
@@ -187,6 +238,7 @@ impl<T> fmt::Display for GlommioError<T> {
                 source, op, path, fd
             ),
             GlommioError::ExecutorError(err) => write!(f, "Executor error: {}", err),
+            GlommioError::BuilderError(err) => write!(f, "Executror builder error: {}", err),
             GlommioError::Closed(rt) => match rt {
                 ResourceType::Semaphore {
                     requested,
@@ -359,6 +411,20 @@ impl<T> Debug for GlommioError<T> {
                     f.write_fmt(format_args!("Invalid Executor ID {{ id: {} }}", x))
                 }
             },
+            GlommioError::BuilderError(kind) => match kind {
+                BuilderErrorKind::InsufficientCpus {
+                    required,
+                    available,
+                } => f.write_fmt(format_args!(
+                    "InsufficientCpus {{ required: {}, available: {} }}",
+                    required, available
+                )),
+                BuilderErrorKind::NrShards { cpu_sets, shards } => f.write_fmt(format_args!(
+                    "NrShards {{ cpu_sets: {}, shards: {} }}",
+                    cpu_sets, shards
+                )),
+                BuilderErrorKind::ThreadPanic(_) => write!(f, "Thread panicked {{ .. }}"),
+            },
             GlommioError::EnhancedIoError {
                 source,
                 op,
@@ -403,6 +469,12 @@ impl<T> From<GlommioError<T>> for io::Error {
                 io::ErrorKind::InvalidInput,
                 format!("invalid executor id {}", id),
             ),
+            GlommioError::BuilderError(BuilderErrorKind::InsufficientCpus { .. })
+            | GlommioError::BuilderError(BuilderErrorKind::NrShards { .. })
+            | GlommioError::BuilderError(BuilderErrorKind::ThreadPanic(_)) => io::Error::new(
+                io::ErrorKind::Other,
+                format!("Executor builder error: {}", display_err),
+            ),
             GlommioError::EnhancedIoError { source, .. } => {
                 io::Error::new(source.kind(), display_err)
             }
@@ -588,8 +660,7 @@ mod test {
             None,
             Some(32),
         );
-        Err(enhanced)?;
-        Ok(())
+        Err(enhanced.into())
     }
 
     #[test]
