@@ -22,8 +22,9 @@ use futures_lite::{
 };
 use std::{
     cell::RefCell,
-    cmp::Reverse,
+    collections::VecDeque,
     io,
+    os::unix::prelude::AsRawFd,
     pin::Pin,
     rc::Rc,
     task::{Context, Poll},
@@ -47,7 +48,7 @@ pub struct DmaStreamReaderBuilder {
     end: u64,
     buffer_size: usize,
     read_ahead: usize,
-    file: Rc<DmaFile>,
+    pub(super) file: Rc<DmaFile>,
 }
 
 impl DmaStreamReaderBuilder {
@@ -371,7 +372,7 @@ pub struct DmaStreamReader {
 macro_rules! collect_error {
     ( $state:expr, $res:expr ) => {{
         if let Err(x) = $res {
-            $state.borrow_mut().error = Some(x.into());
+            $state.error = Some(x.into());
             true
         } else {
             false
@@ -407,8 +408,10 @@ impl DmaStreamReader {
         let to_cancel = handles.len();
         let cancelled = stream::iter(handles).then(|f| f).count().await;
         assert_eq!(to_cancel, cancelled);
-        let res = self.file.close_rc().await;
-        collect_error!(self.state, res);
+
+        // the file may be used by other processes so failing to close it is not an
+        // issue.
+        let _ = self.file.close_rc().await;
 
         let mut state = self.state.borrow_mut();
         match state.error.take() {
@@ -699,7 +702,7 @@ impl AsyncRead for DmaStreamReader {
 pub struct DmaStreamWriterBuilder {
     buffer_size: usize,
     write_behind: usize,
-    flush_on_close: bool,
+    sync_on_close: bool,
     file: Rc<DmaFile>,
 }
 
@@ -733,7 +736,7 @@ impl DmaStreamWriterBuilder {
         DmaStreamWriterBuilder {
             buffer_size: 128 << 10,
             write_behind: 4,
-            flush_on_close: true,
+            sync_on_close: true,
             file: Rc::new(file),
         }
     }
@@ -754,7 +757,7 @@ impl DmaStreamWriterBuilder {
     /// Does not issue a sync operation when closing the file. This is dangerous
     /// and in most cases may lead to data loss.
     pub fn with_sync_on_close_disabled(mut self, flush_disabled: bool) -> Self {
-        self.flush_on_close = !flush_disabled;
+        self.sync_on_close = !flush_disabled;
         self
     }
 
@@ -784,24 +787,122 @@ enum FileStatus {
     Closed,  // It was closed, but the poll
 }
 
+#[derive(Debug)]
+enum FlushStatus {
+    Pending(Option<task::JoinHandle<()>>),
+    Complete,
+}
+
+#[derive(Debug)]
+struct DmaStreamWriterFlushState {
+    flushes: VecDeque<(u64, FlushStatus)>,
+    pending_flush_count: usize,
+    flushed_pos: u64,
+}
+
+impl DmaStreamWriterFlushState {
+    fn new(write_behind: usize) -> DmaStreamWriterFlushState {
+        DmaStreamWriterFlushState {
+            flushes: VecDeque::with_capacity(write_behind * 2),
+            pending_flush_count: 0,
+            flushed_pos: 0,
+        }
+    }
+
+    fn last_flush_pos(&self) -> u64 {
+        self.flushes
+            .back()
+            .map_or(self.flushed_pos, |(pos, _)| *pos)
+    }
+
+    fn take_pending_handles(&mut self) -> Vec<task::JoinHandle<()>> {
+        let mut handles = Vec::with_capacity(self.pending_flush_count);
+        for (_, status) in &mut self.flushes {
+            if let FlushStatus::Pending(handle) = status {
+                if let Some(h) = handle.take() {
+                    handles.push(h);
+                }
+            }
+        }
+        handles
+    }
+
+    fn on_start(&mut self, flush_pos: u64, handle: task::JoinHandle<()>) {
+        self.pending_flush_count += 1;
+        assert!(self.last_flush_pos() < flush_pos);
+        self.flushes
+            .push_back((flush_pos, FlushStatus::Pending(Some(handle))));
+    }
+
+    fn on_complete(&mut self, flush_pos: u64) -> u64 {
+        self.pending_flush_count -= 1;
+        // note:
+        // - writes can complete out-of-order
+        // - `flushes` is maintained in sorted order of position
+        // we will update the status of `flush_pos` to complete.
+        // we also count contiguous completed flushes from the front,
+        // `flushed_pos` can be updated accordingly and we don't
+        // need to track them anymore, so drain that range.
+        let mut drainable_len = 0usize;
+        let mut counting = true;
+        for (pos, status) in &mut self.flushes {
+            if *pos == flush_pos {
+                *status = FlushStatus::Complete;
+            }
+            if counting {
+                if let FlushStatus::Complete = status {
+                    drainable_len += 1;
+                    self.flushed_pos = *pos;
+                } else {
+                    counting = false;
+                }
+            } else if *pos >= flush_pos {
+                break;
+            }
+        }
+        if drainable_len > 0 {
+            self.flushes.drain(..drainable_len);
+        }
+        self.flushed_pos
+    }
+}
+
 #[derive(Debug)]
 struct DmaStreamWriterState {
     buffer_size: usize,
     waker: Option<Waker>,
     file_status: FileStatus,
     error: Option<io::Error>,
-    pending: AHashMap<u64, task::JoinHandle<()>>,
+    flush_state: DmaStreamWriterFlushState,
     current_buffer: Option<DmaBuffer>,
-    file_pos: u64,
-    flush_id: u64,
-    // this is so we track the last flushed pos. Buffers may return
-    // out of order, so we can't report them as flushed_pos yet. Store for
-    // later.
-    out_of_order_write_returns: Vec<u64>,
-    flushed_pos: u64,
+    aligned_pos: u64,
+    synced_pos: u64,
     buffer_pos: usize,
     write_behind: usize,
-    flush_on_close: bool,
+    sync_on_close: bool,
+}
+
+macro_rules! already_closed {
+    () => {
+        // We do it like this so that the write and read close errors are exactly the
+        // same. The reader uses enhanced errors, so it has a message in
+        // the inner attribute of io::Error
+        Poll::Ready(Err(io::Error::new(
+            io::ErrorKind::Other,
+            format!("{}", io::Error::from_raw_os_error(libc::EBADF)),
+        )))
+    };
+}
+
+macro_rules! ensure_not_closed {
+    ( $file:expr ) => {
+        match $file.take() {
+            Some(file) => file,
+            None => {
+                return already_closed!();
+            }
+        }
+    };
 }
 
 impl DmaStreamWriterState {
@@ -811,115 +912,148 @@ impl DmaStreamWriterState {
         self.waker = Some(waker);
     }
 
-    fn initiate_close(&mut self, waker: Waker, state: Rc<RefCell<Self>>, file: Rc<DmaFile>) {
-        let final_pos = self.file_pos();
-        let must_truncate = final_pos != self.file_pos;
-
-        if self.buffer_pos > 0 {
-            let buffer = self.current_buffer.take();
-            self.flush_one_buffer(buffer.unwrap(), state.clone(), file.clone());
-            if must_truncate {
-                // flush will have adjusted that, we will fix.
-                self.file_pos = final_pos;
-            }
+    fn must_truncate(&self) -> bool {
+        if let FileStatus::Closed = self.file_status {
+            return false;
         }
-        let mut drainers = std::mem::take(&mut self.pending);
-        let flush_on_close = self.flush_on_close;
+        self.flushed_pos() > self.aligned_pos
+    }
+
+    fn initiate_close(
+        &mut self,
+        waker: Waker,
+        state: Rc<RefCell<Self>>,
+        file: Rc<DmaFile>,
+        do_close: bool,
+    ) {
         self.file_status = FileStatus::Closing;
+        let final_pos = self.current_pos();
+        self.flush_padded(state.clone(), file.clone());
+        let mut pending = self.take_pending_handles();
         Local::local(async move {
             defer! {
                 waker.wake();
             }
 
-            for (_, v) in drainers.drain() {
-                v.await;
+            for flush in pending.drain(..) {
+                flush.await;
             }
-            if state.borrow().error.is_some() {
+
+            // safe to borrow now across yield points as no more flushers
+            let mut state = state.borrow_mut();
+
+            if state.error.is_some() {
                 return;
             }
 
-            if flush_on_close {
+            assert_eq!(state.flushed_pos(), final_pos);
+
+            if state.sync_on_close && state.synced_pos < final_pos {
                 let res = file.fdatasync().await;
                 if collect_error!(state, res) {
                     return;
                 }
+                state.synced_pos = final_pos;
             }
 
-            if must_truncate {
+            if state.must_truncate() {
                 let res = file.truncate(final_pos).await;
                 if collect_error!(state, res) {
                     return;
                 }
             }
 
-            let res = file.close_rc().await;
-            collect_error!(state, res);
-
-            let mut state = state.borrow_mut();
-            state.flushed_pos = final_pos;
-            drop(state);
+            if do_close {
+                let res = file.close_rc().await;
+                collect_error!(state, res);
+            }
         })
         .detach();
     }
 
-    fn file_pos(&self) -> u64 {
-        self.file_pos + self.buffer_pos as u64
+    fn current_pos(&self) -> u64 {
+        self.aligned_pos + self.buffer_pos as u64
     }
 
     fn flushed_pos(&self) -> u64 {
-        self.flushed_pos
+        self.flush_state.flushed_pos
     }
 
-    fn adjust_flushed_pos(&mut self, just_written: u64) {
-        self.out_of_order_write_returns.push(just_written);
-        self.out_of_order_write_returns.sort_by_key(|x| Reverse(*x));
-        while let Some(x) = self.out_of_order_write_returns.last() {
-            if *x == self.flushed_pos {
-                self.flushed_pos += self.buffer_size as u64;
-                self.out_of_order_write_returns.pop();
-            } else {
-                return;
-            }
-        }
+    fn pending_flush_count(&mut self) -> usize {
+        self.flush_state.pending_flush_count
     }
 
-    fn current_pending(&mut self) -> Vec<task::JoinHandle<()>> {
-        let mut handles = Vec::new();
-        for (_k, v) in self.pending.drain() {
-            handles.push(v);
+    fn take_pending_handles(&mut self) -> Vec<task::JoinHandle<()>> {
+        self.flush_state.take_pending_handles()
+    }
+
+    fn flush_padded(&mut self, state: Rc<RefCell<Self>>, file: Rc<DmaFile>) -> bool {
+        if self.buffer_pos == 0 {
+            return false;
+        } else {
+            assert!(
+                self.buffer_pos < self.buffer_size,
+                "full buffer should have already been flushed"
+            );
         }
-        handles
+        let last_flush_pos = self.flush_state.last_flush_pos();
+        let current_pos = self.current_pos();
+        if last_flush_pos == current_pos {
+            return false;
+        } else {
+            assert!(last_flush_pos < current_pos);
+        }
+        let buffer = self.current_buffer.take().unwrap();
+        if let FileStatus::Open = self.file_status {
+            let mut buffer_copy = file.alloc_dma_buffer(self.buffer_size);
+            buffer_copy.as_bytes_mut()[..self.buffer_pos]
+                .copy_from_slice(&buffer.as_bytes()[..self.buffer_pos]);
+            self.current_buffer = Some(buffer_copy);
+        }
+        self.flush_one_buffer(buffer, state, file);
+        true
     }
 
     fn flush_one_buffer(&mut self, buffer: DmaBuffer, state: Rc<RefCell<Self>>, file: Rc<DmaFile>) {
-        let file_pos = self.file_pos;
-        self.file_pos += self.buffer_size as u64;
-        self.buffer_pos = 0;
-        let flush_id = self.flush_id;
-        self.flush_id += 1;
-        self.pending.insert(
-            flush_id,
-            Local::local(async move {
-                let res = file.write_at(buffer, file_pos).await;
-                collect_error!(state, res);
-
-                let mut state = state.borrow_mut();
-                state.adjust_flushed_pos(file_pos);
-                state.pending.remove(&flush_id); // can be None during close
-                if let Some(waker) = state.waker.take() {
-                    drop(state);
-                    waker.wake();
-                }
-            })
-            .detach(),
-        );
+        let aligned_pos = self.aligned_pos;
+        let flush_pos = self.current_pos();
+        let handle = Local::local(async move {
+            let res = file.write_at(buffer, aligned_pos).await;
+            let mut state = state.borrow_mut();
+            if !collect_error!(state, res) {
+                state.flush_state.on_complete(flush_pos);
+            }
+            if let Some(waker) = state.waker.take() {
+                drop(state);
+                waker.wake();
+            }
+        })
+        .detach();
+        self.flush_state.on_start(flush_pos, handle);
     }
 }
 
 impl Drop for DmaStreamWriterState {
     fn drop(&mut self) {
-        for (_, v) in self.pending.drain() {
-            v.cancel();
+        assert!(
+            self.take_pending_handles().is_empty(),
+            "DmaStreamerWriter::drop() should have cancelled pending flushes"
+        );
+    }
+}
+
+impl Drop for DmaStreamWriter {
+    fn drop(&mut self) {
+        let mut state = self.state.borrow_mut();
+        let mut pending = state.take_pending_handles();
+        for flush in pending.drain(..) {
+            flush.cancel();
+        }
+        if state.must_truncate() {
+            let file = self.file.take().unwrap();
+            Local::get_reactor()
+                .sys
+                .async_truncate(file.as_raw_fd(), state.flushed_pos());
         }
     }
 }
@@ -951,17 +1085,15 @@ impl DmaStreamWriter {
         let state = DmaStreamWriterState {
             buffer_size: builder.buffer_size,
             write_behind: builder.write_behind,
-            flush_on_close: builder.flush_on_close,
+            sync_on_close: builder.sync_on_close,
             current_buffer: None,
             waker: None,
-            pending: AHashMap::new(),
+            flush_state: DmaStreamWriterFlushState::new(builder.write_behind),
             error: None,
             file_status: FileStatus::Open,
-            file_pos: 0,
+            aligned_pos: 0,
             buffer_pos: 0,
-            flushed_pos: 0,
-            flush_id: 0,
-            out_of_order_write_returns: Vec::new(),
+            synced_pos: 0,
         };
 
         let state = Rc::new(RefCell::new(state));
@@ -994,7 +1126,7 @@ impl DmaStreamWriter {
     ///
     /// [`DmaStreamWriter`]: struct.DmaStreamWriter.html
     pub fn current_pos(&self) -> u64 {
-        self.state.borrow().file_pos()
+        self.state.borrow().current_pos()
     }
 
     /// Acquires the current position of this [`DmaStreamWriter`] that is
@@ -1040,13 +1172,86 @@ impl DmaStreamWriter {
         self.state.borrow().flushed_pos()
     }
 
-    /// Waits for all currently in-flight buffers to return and be safely stored
-    /// in the underlying storage
+    async fn flush_inner(&self, partial: bool) -> Result<u64> {
+        let (target_pos, mut pending) = {
+            let mut state = self.state.borrow_mut();
+            let pos = if partial && state.buffer_pos > 0 {
+                state.flush_padded(self.state.clone(), self.file.clone().unwrap());
+                state.current_pos()
+            } else {
+                state.aligned_pos
+            };
+            (pos, state.take_pending_handles())
+        };
+        for flush in pending.drain(..) {
+            flush.await;
+        }
+        let mut state = self.state.borrow_mut();
+        if let Some(err) = current_error!(state) {
+            return err;
+        }
+        assert!(state.flushed_pos() >= target_pos);
+        Ok(state.flushed_pos())
+    }
+
+    async fn sync_inner(&self) -> Result<u64> {
+        let (flushed_pos, presync_pos) = {
+            let state = self.state.borrow();
+            (state.flushed_pos(), state.synced_pos)
+        };
+        if presync_pos < flushed_pos {
+            self.file.clone().unwrap().fdatasync().await?;
+            self.state.borrow_mut().synced_pos = flushed_pos;
+        } else {
+            assert_eq!(presync_pos, flushed_pos);
+        }
+        Ok(flushed_pos)
+    }
+
+    /// Waits for all currently in-flight buffers to be written to the
+    /// underlying storage.
     ///
-    /// Note that the current buffer being written to is not flushed, as it may
-    /// not be properly aligned. Buffers that are currently in-flight will
-    /// be waited on, and a sync operation will be issued by the operating
-    /// system.
+    /// This does not include the current buffer if it is not full. If all data
+    /// must be flushed, use [`flush`].
+    ///
+    /// Returns the flushed position of the file.
+    ///
+    /// # Examples
+    /// ```no_run
+    /// use futures::io::AsyncWriteExt;
+    /// use glommio::{
+    ///     io::{DmaFile, DmaStreamWriterBuilder},
+    ///     LocalExecutor,
+    /// };
+    ///
+    /// let ex = LocalExecutor::default();
+    /// ex.run(async {
+    ///     let file = DmaFile::create("myfile.txt").await.unwrap();
+    ///     let mut writer = DmaStreamWriterBuilder::new(file)
+    ///         .with_buffer_size(4096)
+    ///         .with_write_behind(2)
+    ///         .build();
+    ///     let buffer = [0u8; 5000];
+    ///     writer.write_all(&buffer).await.unwrap();
+    ///     // with 5000 bytes written into a 4096-byte buffer a flush
+    ///     // has certainly started. But if very likely didn't finish right
+    ///     // away.
+    ///     assert_eq!(writer.current_flushed_pos(), 0);
+    ///     assert_eq!(writer.flush_aligned().await.unwrap(), 4096);
+    ///     writer.close().await.unwrap();
+    /// });
+    /// ```
+    ///
+    /// [`flush`]: https://docs.rs/futures/0.3.15/futures/io/trait.AsyncWriteExt.html#method.flush
+    pub async fn flush_aligned(&self) -> Result<u64> {
+        self.flush_inner(false).await
+    }
+
+    /// Waits for all currently in-flight buffers to be written to the
+    /// underlying storage, and ensures they are safely persisted.
+    ///
+    /// This does not include the current buffer if it is not full. If all data
+    /// must be synced, use [`Self::sync`].
     ///
     /// Returns the flushed position of the file at the time the sync started.
     ///
@@ -1069,26 +1274,90 @@ impl DmaStreamWriter {
     ///     writer.write_all(&buffer).await.unwrap();
     ///     // with 5000 bytes written into a 4096-byte buffer a flush
     ///     // has certainly started. But if very likely didn't finish right
-    ///     // away. It will not be reflected on current_flushed_pos(), but a
-    ///     // sync() will wait on it.
+    ///     // away.
     ///     assert_eq!(writer.current_flushed_pos(), 0);
-    ///     assert_eq!(writer.sync().await.unwrap(), 4096);
+    ///     assert_eq!(writer.sync_aligned().await.unwrap(), 4096);
     ///     writer.close().await.unwrap();
     /// });
     /// ```
+    pub async fn sync_aligned(&self) -> Result<u64> {
+        self.flush_aligned().await?;
+        self.sync_inner().await
+    }
+
+    /// Waits for all buffers to be written to the underlying storage, and
+    /// ensures they are safely persisted.
+    ///
+    /// This includes the current buffer even if it is not full, by padding it.
+    /// The padding will get over-written by future writes, or truncated upon
+    /// [`close`].
+    ///
+    /// Returns the flushed position of the file at the time the sync started.
+    ///
+    /// # Examples
+    /// ```no_run
+    /// use futures::io::AsyncWriteExt;
+    /// use glommio::{
+    ///     io::{DmaFile, DmaStreamWriterBuilder},
+    ///     LocalExecutor,
+    /// };
+    ///
+    /// let ex = LocalExecutor::default();
+    /// ex.run(async {
+    ///     let file = DmaFile::create("myfile.txt").await.unwrap();
+    ///     let mut writer = DmaStreamWriterBuilder::new(file)
+    ///         .with_buffer_size(4096)
+    ///         .with_write_behind(2)
+    ///         .build();
+    ///     let buffer = [0u8; 5000];
+    ///     writer.write_all(&buffer).await.unwrap();
+    ///     // with 5000 bytes written into a 4096-byte buffer a flush
+    ///     // has certainly started. But if very likely didn't finish right
+    ///     // away.
+    ///     assert_eq!(writer.current_flushed_pos(), 0);
+    ///     assert_eq!(writer.sync().await.unwrap(), 5000);
+    ///     writer.close().await.unwrap();
+    /// });
+    /// ```
+    ///
+    /// [`close`]: https://docs.rs/futures/0.3.15/futures/io/trait.AsyncWriteExt.html#method.close
     pub async fn sync(&self) -> Result<u64> {
-        let (mut pending, file_pos_at_sync_time) = {
-            let mut state = self.state.borrow_mut();
-            (state.current_pending(), state.file_pos)
-        };
+        self.flush_inner(true).await?;
+        self.sync_inner().await
+    }
 
-        for v in pending.drain(..) {
-            v.await;
+    // internal function that does everything that close does (flushes buffers, etc,
+    // but leaves the file open. Useful for the immutable file abstraction.
+    pub(super) fn poll_seal(
+        &mut self,
+        cx: &mut Context<'_>,
+    ) -> Poll<io::Result<DmaStreamReaderBuilder>> {
+        let mut state = self.state.borrow_mut();
+        match state.file_status {
+            FileStatus::Open => {
+                let file = ensure_not_closed!(self.file.clone());
+                state.initiate_close(cx.waker().clone(), self.state.clone(), file, false);
+                Poll::Pending
+            }
+            FileStatus::Closing => {
+                state.file_status = FileStatus::Closed;
+                let file = ensure_not_closed!(self.file);
+
+                let wb = state.write_behind;
+                let bufsz = state.buffer_size;
+                let builder = DmaStreamReaderBuilder::from_rc(file)
+                    .with_read_ahead(wb)
+                    .with_buffer_size(bufsz);
+
+                match current_error!(state) {
+                    Some(err) => Poll::Ready(err),
+                    None => Poll::Ready(Ok(builder)),
+                }
+            }
+            FileStatus::Closed => {
+                already_closed!()
+            }
         }
-
-        let file = self.file.clone().unwrap();
-        file.fdatasync().await?;
-        Ok(file_pos_at_sync_time)
     }
 }
 
@@ -1107,7 +1376,7 @@ impl AsyncWrite for DmaStreamWriter {
         while written < buf.len() {
             match state.current_buffer.take() {
                 None => {
-                    if state.pending.len() < state.write_behind {
+                    if state.pending_flush_count() < state.write_behind {
                         state.current_buffer = Some(
                             self.file
                                 .as_ref()
@@ -1128,6 +1397,8 @@ impl AsyncWrite for DmaStreamWriter {
                             self.state.clone(),
                             self.file.clone().unwrap(),
                         );
+                        state.aligned_pos += state.buffer_size as u64;
+                        state.buffer_pos = 0;
                     } else {
                         state.current_buffer = Some(buffer);
                     }
@@ -1143,8 +1414,17 @@ impl AsyncWrite for DmaStreamWriter {
         }
     }
 
-    fn poll_flush(self: Pin<&mut Self>, _cx: &mut Context<'_>) -> Poll<io::Result<()>> {
-        Poll::Ready(Ok(()))
+    fn poll_flush(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<io::Result<()>> {
+        let mut state = self.state.borrow_mut();
+        if let Some(err) = current_error!(state) {
+            return Poll::Ready(err);
+        }
+        if state.flushed_pos() == state.current_pos() {
+            return Poll::Ready(Ok(()));
+        }
+        state.flush_padded(self.state.clone(), self.file.clone().unwrap());
+        state.add_waker(cx.waker().clone());
+        Poll::Pending
     }
 
     fn poll_close(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<io::Result<()>> {
@@ -1152,7 +1432,7 @@ impl AsyncWrite for DmaStreamWriter {
         let mut state = self.state.borrow_mut();
         match state.file_status {
             FileStatus::Open => {
-                state.initiate_close(cx.waker().clone(), self.state.clone(), file.unwrap());
+                state.initiate_close(cx.waker().clone(), self.state.clone(), file.unwrap(), true);
                 Poll::Pending
             }
             FileStatus::Closing => {
@@ -1163,13 +1443,7 @@ impl AsyncWrite for DmaStreamWriter {
                 }
             }
             FileStatus::Closed => {
-                // We do it like this so that the write and read close errors are exactly the
-                // same. The reader uses enhanced errors, so it has a message in
-                // the inner attribute of io::Error
-                Poll::Ready(Err(io::Error::new(
-                    io::ErrorKind::Other,
-                    format!("{}", io::Error::from_raw_os_error(libc::EBADF)),
-                )))
+                already_closed!()
             }
         }
     }
@@ -1183,7 +1457,7 @@ mod test {
         timer::Timer,
     };
     use futures::{AsyncReadExt, AsyncWriteExt};
-    use std::{io::ErrorKind, time::Duration};
+    use std::{io::ErrorKind, path::Path, time::Duration};
 
     macro_rules! file_stream_read_test {
         ( $name:ident, $dir:ident, $kind:ident, $file:ident, $file_size:ident: $size:tt, $code:block) => {
@@ -1243,6 +1517,10 @@ mod test {
         };
     }
 
+    fn file_size<P: AsRef<Path>>(path: P) -> u64 {
+        std::fs::metadata(path).unwrap().len()
+    }
+
     file_stream_read_test!(read_exact_empty_file, path, _k, file, _file_size: 0, {
         let mut reader = DmaStreamReaderBuilder::new(file)
             .build();
@@ -1364,8 +1642,7 @@ mod test {
             .with_buffer_size(1 << 10)
             .build();
 
-        let mut buf = Vec::with_capacity(file_size);
-        buf.resize(file_size, 0);
+        let mut buf = vec![0; file_size];
         reader.read_exact(&mut buf).await.unwrap();
         check_contents!(buf, 0);
         reader.close().await.unwrap();
@@ -1731,6 +2008,24 @@ mod test {
         assert_eq!(writer.current_flushed_pos(), 5000);
     });
 
+    file_stream_write_test!(flush_and_drop, path, _k, filename, file, {
+        let mut writer = DmaStreamWriterBuilder::new(file)
+            .with_buffer_size(4096)
+            .with_write_behind(2)
+            .build();
+
+        assert_eq!(writer.current_pos(), 0);
+        let buffer = [0u8; 5000];
+        writer.write_all(&buffer).await.unwrap();
+        assert_eq!(writer.flush_aligned().await.unwrap(), 4096);
+        assert_eq!(file_size(&filename), 4096);
+        writer.flush().await.unwrap();
+        assert_eq!(writer.current_flushed_pos(), 5000);
+        assert_eq!(file_size(&filename), 8192);
+        drop(writer);
+        assert_eq!(file_size(&filename), 5000);
+    });
+
     file_stream_write_test!(sync_and_close, path, _k, filename, file, {
         let mut writer = DmaStreamWriterBuilder::new(file)
             .with_buffer_size(4096)
@@ -1740,11 +2035,48 @@ mod test {
         assert_eq!(writer.current_pos(), 0);
         let buffer = [0u8; 5000];
         writer.write_all(&buffer).await.unwrap();
-        assert_eq!(writer.sync().await.unwrap(), 4096);
+        assert_eq!(writer.sync().await.unwrap(), 5000);
+        assert_eq!(file_size(&filename), 8192);
         // write more
         writer.write_all(&buffer).await.unwrap();
         writer.close().await.unwrap();
 
         assert_eq!(writer.current_flushed_pos(), 10000);
+        assert_eq!(file_size(&filename), 10000);
     });
+
+    #[test]
+    fn flushstate() {
+        test_executor!(async move {
+            let handle_gen = || Local::local(async move {}).detach();
+            let mut state = DmaStreamWriterFlushState::new(4);
+
+            state.on_start(8, handle_gen());
+            // flushes: [8->Pending(Some)]
+            assert_eq!(state.take_pending_handles().len(), 1);
+            // flushes: [8->Pending(None)]
+            assert_eq!(state.take_pending_handles().len(), 0);
+            // flushes: [8->Pending(None)]
+            assert_eq!(state.flushes.len(), 1);
+            assert_eq!(state.pending_flush_count, 1);
+            assert_eq!(state.on_complete(8), 8);
+            // flushes: []
+            assert_eq!(state.flushes.len(), 0);
+
+            state.on_start(16, handle_gen());
+            state.on_start(24, handle_gen());
+            state.on_start(32, handle_gen());
+            // flushes: [16->Pending, 24->Pending, 32->Pending]
+            assert_eq!(state.flushes.len(), 3);
+            assert_eq!(state.on_complete(24), 8);
+            // flushes: [16->Pending, 24->Complete, 32->Pending]
+            assert_eq!(state.on_complete(32), 8);
+            assert_eq!(state.flushes.len(), 3);
+            assert_eq!(state.pending_flush_count, 1);
+            // flushes: [16->Pending, 24->Complete, 32->Complete]
+            assert_eq!(32, state.on_complete(16));
+            // flushes: []
+            assert_eq!(state.flushes.len(), 0);
+        });
+    }
 }
