@@ -306,7 +306,7 @@ macro_rules! wake {
     ($waker:expr $(,)?) => {
         use log::error;
 
-        if let Err(x) = panic::catch_unwind(|| $waker.wake()) {
+        if let Err(x) = std::panic::catch_unwind(|| $waker.wake()) {
             error!("Panic while calling waker! {:?}", x);
         }
     };
@@ -442,8 +442,17 @@ pub mod timer;
 
 pub use crate::{
     byte_slice_ext::{ByteSliceExt, ByteSliceMutExt},
-    error::{GlommioError, QueueErrorKind, ResourceType, Result},
+    error::{
+        BuilderErrorKind,
+        ExecutorErrorKind,
+        GlommioError,
+        QueueErrorKind,
+        ReactorErrorKind,
+        ResourceType,
+        Result,
+    },
     executor::{
+        CpuSet,
         ExecutorStats,
         LocalExecutor,
         LocalExecutorBuilder,
@@ -456,6 +465,7 @@ pub use crate::{
         TaskQueueStats,
     },
     shares::{Shares, SharesManager},
+    sys::hardware_topology::CpuLocation,
 };
 pub use enclose::enclose;
 pub use scopeguard::defer;
@@ -545,6 +555,8 @@ pub struct RingIoStats {
     pub(crate) file_bytes_read: u64,
     pub(crate) file_buffered_reads: u64,
     pub(crate) file_buffered_bytes_read: u64,
+    pub(crate) file_deduped_reads: u64,
+    pub(crate) file_deduped_bytes_read: u64,
     pub(crate) file_writes: u64,
     pub(crate) file_bytes_written: u64,
     pub(crate) file_buffered_writes: u64,
@@ -583,6 +595,13 @@ impl RingIoStats {
         (self.file_reads, self.file_bytes_read)
     }
 
+    /// File read IO stats (deduplicated)
+    ///
+    /// Returns the number of reads that fed from another preexisting buffer
+    pub fn file_deduped_reads(&self) -> (u64, u64) {
+        (self.file_deduped_reads, self.file_deduped_bytes_read)
+    }
+
     /// Buffered file read IO stats
     ///
     /// Returns the number of individual buffered read ops as well as bytes read
@@ -615,6 +634,8 @@ impl Sum<RingIoStats> for RingIoStats {
             file_bytes_read: a.file_bytes_read + b.file_bytes_read,
             file_buffered_reads: a.file_buffered_reads + b.file_buffered_reads,
             file_buffered_bytes_read: a.file_buffered_bytes_read + b.file_buffered_bytes_read,
+            file_deduped_reads: a.file_deduped_reads + b.file_deduped_reads,
+            file_deduped_bytes_read: a.file_deduped_bytes_read + b.file_deduped_bytes_read,
             file_writes: a.file_writes + b.file_writes,
             file_bytes_written: a.file_bytes_written + b.file_bytes_written,
             file_buffered_writes: a.file_buffered_writes + b.file_buffered_writes,
@@ -658,6 +679,8 @@ pub(crate) mod test_utils {
     use super::*;
     use nix::sys::statfs::*;
     use std::path::{Path, PathBuf};
+    use tracing::{debug, error, info, trace, warn};
+    use tracing_subscriber::EnvFilter;
 
     #[derive(Copy, Clone)]
     pub(crate) enum TestDirectoryKind {
@@ -709,4 +732,18 @@ pub(crate) mod test_utils {
         };
         TestDirectory { path: dir, kind }
     }
+
+    #[test]
+    #[allow(unused_must_use)]
+    fn test_tracing_init() {
+        tracing_subscriber::fmt::fmt()
+            .with_env_filter(EnvFilter::from_env("GLOMMIO_TRACE"))
+            .try_init();
+
+        info!("Started tracing..");
+        debug!("Started tracing..");
+        warn!("Started tracing..");
+        trace!("Started tracing..");
+        error!("Started tracing..");
+    }
 }
